/*
* ----------------------------------------------------------------------------
* "THE BEER-WARE LICENSE" (Revision 42):
* <timklge@gmail.com> wrote this file. As long as you retain this notice you
* can do whatever you want with this stuff. If we meet some day, and you think
* this stuff is worth it, you can buy me a beer in return - Tim Kluge
* ----------------------------------------------------------------------------
*/

/**
 * Tiny library that provides functionality to encode and decode base58 values.
 */
library base58;

import "dart:math";
import "dart:convert";

/// Alphabet used by base58. Contains all characters used by base64 excluding some that may
/// be misunderstanding (0, O, I, l)
const String base58_characters = '123456789abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ';

class Base58Encoder extends Converter<int, String> {
  String convert(int value){
    final encoded = new StringBuffer();
    final List<String> chars = new List();
    do {
      chars.insert(0, base58_characters[value % 58]);
      value = value ~/ 58;
    } while(value > 0);
    
    encoded.writeAll(chars);
    return encoded.toString();
  }
}

class Base58Decoder extends Converter<String, int> {
  int convert(String value){
    int decoded = 0;
    for(int i = value.length - 1; i >= 0; i--){
      decoded += pow(58, value.length - 1 - i) 
          * base58_characters.indexOf(value[i]);
    }
    return decoded;
  }
}

/// A codec that encodes numbers to base58-encoded strings and decodes base58-encoded strings to numbers.
class Base58Codec extends Codec<int, String> {
  final encoder = new Base58Encoder();
  final decoder = new Base58Decoder();
}
